/*
 * indicator-network - user interface for connman
 * Copyright 2010-2011 Canonical Ltd.
 *
 * Authors:
 * Andrew Higginson
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class EditConnectionDialog : GLib.Object {

    public signal void response(int type);

    private WifiConnection connection;
    private Gtk.Builder builder;

    private Gtk.Window dialog;
    private Gtk.Table table_main;
    private Gtk.Button button_save;
    private Gtk.Button button_cancel;
    private Gtk.Entry entry_name;
    private Gtk.Entry entry_ssid;
    private Gtk.Entry entry_bssid;
    private Gtk.ComboBox combobox_mode;
    private Gtk.CheckButton checkbutton_autoconnect;
    private Gtk.CheckButton checkbutton_share_settings;

    private string[] hint_labels = {"label_hint_name", "label_hint_ssid",
                                    "label_hint_bssid","label_hint_mode"};

    public EditConnectionDialog(WifiConnection connection, string datadir) {
        this.connection = connection;

        /// Creation
        this.builder = Utils.Gui.new_builder(datadir+"ui/edit_connection_dialog.ui");
        this.get_widgets();
        this.create_extra_widgets();
        
        this.connect_signals();
        this.update_widget_values();
    }

    private void create_extra_widgets() {
        // Creation
        this.combobox_mode = new Gtk.ComboBox.text();

        // Population
        WirelessMode[] wireless_modes = {WirelessMode.INFRASTRUCTURE, WirelessMode.ADHOC};
        foreach (var mode in wireless_modes) {
            this.combobox_mode.append_text(mode.get_name());
        }

        // Packing
        this.table_main.attach(this.combobox_mode, 1, 2, 6, 7, Gtk.AttachOptions.FILL, Gtk.AttachOptions.FILL, 0, 0);

        // Setup hint labels
        foreach (var hint_label in this.hint_labels) {
            var hint_label_widget = this.builder.get_object(hint_label) as Gtk.Label;
            hint_label_widget.realize.connect(() => {
                                                    var state = hint_label_widget.get_state();
                                                    var color = hint_label_widget.style.fg[state];
                                                    var new_color = Utils.Color.shade(color);
                                                    hint_label_widget.modify_fg(state, new_color);
                                                    });
        }
    }

    private void get_widgets() {
        this.dialog = this.builder.get_object("window_main") as Gtk.Window;
        this.table_main = this.builder.get_object("table_main") as Gtk.Table;
        this.button_save = this.builder.get_object("button_save") as Gtk.Button;
        this.button_cancel = this.builder.get_object("button_cancel") as Gtk.Button;
        
        this.entry_name = this.builder.get_object("entry_name") as Gtk.Entry;
        this.entry_ssid = this.builder.get_object("entry_ssid") as Gtk.Entry;
        this.entry_bssid = this.builder.get_object("entry_bssid") as Gtk.Entry;
        this.checkbutton_autoconnect = this.builder.get_object("checkbutton_autoconnect") as Gtk.CheckButton;
        this.checkbutton_share_settings = this.builder.get_object("checkbutton_share_settings") as Gtk.CheckButton;
    }

    private void connect_signals() {
        this.button_save.clicked.connect(this.on_button_save_clicked);
        this.button_cancel.clicked.connect(this.on_button_cancel_clicked);
    }

    private void update_widget_values() {
        var name = this.connection.name;
        var ssid = this.connection.ssid;
        var bssid = this.connection.bssid;
        var mode = this.connection.mode;
        var autoconnect = this.connection.autoconnect;
        var share_settings = this.connection.share_settings;

        if (name == null) {
            this.entry_name.set_text("");
        } else {
            this.entry_name.set_text(name);
        }
        if (ssid == null) {
            this.entry_ssid.set_text("");
        } else {
            this.entry_ssid.set_text(ssid);
        }
        if (bssid == null) {
            this.entry_bssid.set_text("");
        } else {
            this.entry_bssid.set_text(bssid);
        }
        
        this.combobox_mode.set_active(mode);
        this.checkbutton_autoconnect.set_active(autoconnect);
        this.checkbutton_share_settings.set_active(share_settings);
    }
    
    private void save_widget_values() {
        string? name = this.entry_name.get_text();
        string? ssid = this.entry_ssid.get_text();
        string? bssid = this.entry_bssid.get_text();
        WirelessMode mode = (WirelessMode) this.combobox_mode.get_active();
        var autoconnect = this.checkbutton_autoconnect.get_active();
        var share_settings = this.checkbutton_share_settings.get_active();

        if (name == "") {
           name = null;
        } if (ssid == "") {
            ssid = null;
        } if (bssid == "") {
            bssid = null;
        }

        this.connection.name = name;
        this.connection.ssid = ssid;
        this.connection.bssid = bssid;
        this.connection.mode = mode;
        this.connection.autoconnect = autoconnect;
        this.connection.share_settings = share_settings;
    }

    private void on_button_save_clicked() {
        this.save_widget_values();
        this.dialog.destroy();
        this.response(Gtk.ResponseType.APPLY);
    }
    
    private void on_button_cancel_clicked() {
        this.dialog.destroy();
        this.response(Gtk.ResponseType.CANCEL);
    }

    public int run() {
        this.dialog.show_all();
		return 0;
    }
}
